@@ -23,7 +23,7 @@ estimate_profiles_mclust <- function(df, n_profiles, model_numbers, select_vars,
     warnings <- NULL
     no_na_rows <- !apply(df, 1, anyNA)
     if(any(!no_na_rows)){
-        warning("The mclust algorithm does not allow for missing data. Some rows were omitted from analysis. Consider using Mplus, which accounts for cases with partially missing data, or use a non-parametric single imputation technique prior to analysis, such as the R-package 'missForest'.\n")
+        warning("The mclust algorithm does not allow for missing data. Some rows were omitted from analysis. Consider using OpenMx, which accounts for cases with partially missing data, or use a non-parametric single imputation technique prior to analysis, such as the R-package 'missForest'.\n")
     }
     full_data <- df[no_na_rows, , drop = FALSE]
     boot_model_names <- get_modelname(model_numbers)
@@ -44,14 +44,15 @@ estimate_profiles_mclust <- function(df, n_profiles, model_numbers, select_vars,
 
     boot_blrt <- lapply(boot_model_names, function(mod_name){
         Args_boot[["modelName"]] <- mod_name
-        tryCatch({
-            do.call(mclustBootstrapLRT, Args_boot)
-        }, error = function(e) {
-            return(list(obs = rep(NA, max(n_profiles)),
-                        p.value = rep(NA, max(n_profiles))))
+        # tryCatch(do.call(mclustBootstrapLRT, Args_boot), error = function(e){
+        #     tryCatch(
+                do.call(blrt_mclust, Args_boot)
+        #         , error = function(e){
+        #         list(obs = rep(NA, max(n_profiles)),
+        #              p.value = rep(NA, max(n_profiles)))
+        #     })
+        # })
         })
-    })
-
     # Arguments for Mclust ----------------------------------------------------
     Args_mclust <- dots[which(names(dots) %in% c("prior", "control", "initialization", "warn", "verbose"))]
     Args_mclust[["data"]] <- full_data
@@ -80,7 +81,7 @@ estimate_profiles_mclust <- function(df, n_profiles, model_numbers, select_vars,
             out$model$mclustBootstrap <- do.call(MclustBootstrap, Args_mclustbootstrap)
             out$model$LRTS <- ifelse(this_class == 1, NA, boot_blrt[[which(model_numbers == this_model)]]$obs[this_class-1])
             out$model$LRTS_p <- ifelse(this_class == 1, NA, boot_blrt[[which(model_numbers == this_model)]]$p.value[this_class-1])
-            out$fit <- c(Model = this_model, Classes = this_class, calc_fitindices(out$model))
+            out$fit <- cbind(Model = this_model, Classes = this_class, calc_fitindices(out$model, modelname = Args_mclust[["modelName"]]))
             estimates <- estimates(out$model)
             estimates$Model <- this_model
             estimates$Classes <- this_class
