@@ -1,8 +1,8 @@
 #' @title Estimate latent profiles
-#' @description Estimates latent profiles (finite mixture models) using the open
-#' source package \code{\link[mclust:Mclust]{mclust}}, or the commercial program
-#' Mplus (using the R-interface of
-#' \code{\link[MplusAutomation:mplusModeler]{MplusAutomation}}).
+#' @description Estimates latent profiles (finite mixture models) using either
+#' the open source package \code{\link[mclust:Mclust]{mclust}} or
+#' \code{[OpenMx:mxModel]{OpenMx}}, or the commercial program Mplus (using the
+#' R-interface of \code{\link[MplusAutomation:mplusModeler]{MplusAutomation}}).
 #' @param df data.frame of numeric data; continuous indicators are required for
 #' mixture modeling.
 #' @param n_profiles Integer vector of the number of profiles (or mixture
@@ -19,22 +19,26 @@
 #' to an assumption of conditional independence of the indicators); other
 #' options are "equal" (estimate covariances between items, constrained across
 #' profiles), and "varying" (free covariances across profiles).
-#' @param package Character. Which package to use; 'mclust' or
-#' 'MplusAutomation' (requires Mplus to be installed). Default: 'mclust'.
+#' @param package Character. Which package to use; 'OpenMx', 'mclust', or
+#' 'MplusAutomation' (requires Mplus to be installed). Default: 'OpenMx'.
 #' @param select_vars Character. Optional vector of variable names in \code{df},
 #' to be used for model estimation. Defaults to \code{NULL}, which means all
 #' variables in \code{df} are used.
 #' @param ... Additional arguments are passed to the estimating function; i.e.,
+#' \code{\link[OpenMx]{mxRun}},
 #' \code{\link[mclust]{Mclust}}, or \code{\link[MplusAutomation]{mplusModeler}}.
 #' @return A list of class 'tidyLPA'.
 #' @details Six models are currently available in tidyLPA, corresponding to the
-#' most common requirements. These are:
+#' most common requirements. All models estimate the observed variable means
+#' for each class. The remaining parameters are:
 #' \enumerate{
-#' \item Equal variances and covariances fixed to 0
-#' \item Varying variances and covariances fixed to 0
-#' \item Equal variances and equal covariances
-#' \item Varying variances and equal covariances (not able to be fit w/ mclust)
-#' \item Equal variances and varying covariances (not able to be fit w/ mclust)
+#' \item Equal variances across classes; no covariances between observed
+#' variables
+#' \item Varying variances across classes; no covariances between observed
+#' variables
+#' \item Equal variances and equal covariances across classes
+#' \item Varying variances and equal covariances (not available for \code{package = 'mclust'})
+#' \item Equal variances and varying covariances (not available for \code{package = 'mclust'})
 #' \item Varying variances and varying covariances
 #' }
 #'
@@ -42,11 +46,12 @@
 #' in the \code{models} argument (e.g., \code{models = 1}, or
 #' \code{models = c(1, 2, 3)}), or specify the variances/covariances to be
 #' estimated (e.g.,: \code{variances = c("equal", "varying"), covariances =
-#' c("zero", "equal")}). Note that when mclust is used, \code{models =
-#' c(1, 2, 3, 6)} are the only models available.
+#' c("zero", "equal")}). Note that when \code{package = 'mclust'} is used,
+#' \code{models = c(4, 5)} are not available. Use \code{package = 'OpenMx'} or
+#' \code{package = 'Mplus'} to estimate these models.
 #' @examples
-#'
-#' iris_sample <- iris[c(1:4, 51:54, 101:104), ] # to make example run more quickly
+#' # to make example run more quickly
+#' iris_sample <- iris[c(1:10, 51:60, 101:114), ]
 #'
 #' # Example 1:
 #' iris_sample %>%
@@ -69,6 +74,20 @@
 #'                     covariances = c("zero", "zero"))
 #' }
 #'
+#' \donttest{
+#' if(requireNamespace("OpenMx", quietly = TRUE)){
+#' # to make example run more quickly
+#' iris_sample <- iris[c(1:10, 51:60, 101:114), ]
+#'
+#' # Example 1:
+#' library(OpenMx)
+#' iris_rename <- iris_sample
+#' names(iris_sample) <- letters[seq_along(names(iris_sample))]
+#' iris_sample %>%
+#'   subset(select = c("a", "b", "c")) %>%
+#'   estimate_profiles(3, package = "OpenMx") -> res
+#' }
+#' }
 #' @export
 estimate_profiles <- function(df,
                               n_profiles,
@@ -116,6 +135,7 @@ estimate_profiles.numeric <- function(df,
     NextMethod("estimate_profiles", df)
 }
 
+#' @importFrom tidySEM descriptives
 #' @export
 estimate_profiles.default <- function(df,
                                       n_profiles,
@@ -126,7 +146,6 @@ estimate_profiles.default <- function(df,
                                       select_vars = NULL,
                                       ...) {
     # Check deprecated arguments ----------------------------------------------
-
     deprecated_arguments(c(
         "model" = "Instead, use the argument 'models', or 'variances' and 'covariances'.",
         "toreturn" = "A tibble is returned by default in the $df element of the returned output.",
@@ -149,7 +168,10 @@ estimate_profiles.default <- function(df,
     # Backup df
     df_full <- df
     df <- df[, select_vars, drop = FALSE]
-
+    desc <- descriptives(df)
+    if(any(desc$unique < 10)){
+        warning("Latent profile analysis assumes that all variables have a continuous measurement level. Some variables have fewer than 10 unique values, which may violate this assumption. Carefully examine the results, and avoid complex models, e.g.with free (co)variances or many classes. The variables violating this assumption are: ", paste0(desc$name[desc$unique < 10], collapse = ", "))
+    }
     # Deprecated variable selection -------------------------------------------
     check_dots <- match.call(expand.dots = FALSE)$`...`
     if(any(names(df) %in% unlist(check_dots))){
@@ -159,8 +181,8 @@ estimate_profiles.default <- function(df,
     # Screen for legal input --------------------------------------------------
     package <- package[1]
     if(!inherits(package, "character")) stop("Argument package must be a character string.")
-    package <- c("mclust", "mplusautomation")[pmatch(tolower(package), c("mclust", "mplusautomation"))]
-    if(is.na(package)) stop("Argument 'package' did not match a valid package. Use either 'mclust' or 'MplusAutomation'.")
+    package <- c("mclust", "mplusautomation", "openmx")[pmatch(tolower(package), c("mclust", "mplusautomation", "openmx"))]
+    if(is.na(package)) stop("Argument 'package' did not match a valid package. Use either 'mclust', 'MplusAutomation', or 'OpenMx'.")
 
     # Screen df ---------------------------------------------------------------
 
@@ -203,8 +225,8 @@ estimate_profiles.default <- function(df,
 
     out <- switch(package,
            "mplusautomation" = estimate_profiles_mplus2(df_full, n_profiles, model_numbers, select_vars, ...),
+           "openmx" = estimate_profiles_openmx(df_full, n_profiles, model_numbers, select_vars, ...),
            "mclust" = estimate_profiles_mclust(df_full, n_profiles, model_numbers, select_vars, ...))
-
     # Check warnings here
     warnings <- sapply(out, function(x){!is.null(x[["warnings"]])})
     if(any(warnings)){
@@ -277,11 +299,13 @@ get_fit <- function(x, ...) {
     UseMethod("get_fit", x)
 }
 
+#' @importFrom utils getFromNamespace
+bind_list <- getFromNamespace("bind_list", "tidySEM")
 #' @describeIn get_fit Get fit indices for a latent profile analysis with
 #' multiple numbers of classes and models, of class 'tidyLPA'.
 #' @export
 get_fit.tidyLPA <- function(x, ...) {
-    as_tibble(t(sapply(x, `[[`, "fit")))
+    as_tibble(bind_list(lapply(x, `[[`, "fit")))
 }
 
 #' @describeIn get_fit Get fit indices for a single latent profile analysis
@@ -415,26 +439,23 @@ print.tidyLPA <-
              digits = 2,
              na.print = "",
              ...) {
-        fits <- get_fit(x)
+        fits <- as.data.frame(get_fit(x))
         if(all(is.na(fits[, -c(1,2)]))){
             stop("This tidyLPA analysis does not contain any valid results. Most likely, all models failed to converge.", call. = FALSE)
         }
-        dat <- as.matrix(fits[, c("Model", "Classes", stats)])
+        dat <- fits[, c("Model", "Classes", stats)]
         miss_val <- is.na(dat)
+        formatthese <- which(!(sapply(dat, inherits, what = "character") | sapply(dat, is.wholenumber)))
+        dat[formatthese] <- lapply(dat[formatthese], formatC, digits = digits, format = "f")
         #dat$Model <- paste("Model ", dat$Model)
         #sprintf("%-9s", paste0(names(x$fitindices), ":")),
-        dat[, 3:ncol(dat)] <-
-            sapply(dat[, 3:ncol(dat)], formatC, digits = digits, format = "f")
         dat[miss_val] <- ""
-        #rownames(dat) <- ""
         cat("tidyLPA analysis using", paste0(gsub("^tidyProfile\\.", "", class(x[[1]])[1]), ":"), "\n\n")
-
-        prmatrix(dat,
-                 rowlab = rep("", nrow(dat)),
-                 quote = FALSE,
-                 na.print = na.print)
+        print.data.frame(dat, quote = FALSE, digits = digits)
     }
 
+is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)all(  abs(x - round(x)) < tol)
+
 #' @title Print tidyProfile
 #' @description S3 method 'print' for class 'tidyProfile'.
 #' @param x An object of class 'tidyProfile'.
